@@ -29,13 +29,16 @@ import types
 from collections import Counter
 
 
+date_regex = re.compile('^[0-9]{4}-[0-9]{2}-[0-9]{2}$')
+datetime_regex = re.compile('^[0-9]{4}-[0-9]{2}-[0-9]{2} '
+                            '[0-9]{2}:[0-9]{2}:[0-9]{2}$')
+
 def _str_decode(element, codec):
     if isinstance(element, str):
         return element.decode(codec)
     else:
         return element
 
-
 def _unicode_encode(element, codec):
     if isinstance(element, unicode):
         return element.encode(codec)
@@ -188,41 +191,45 @@ class Table(object):
         The types are identified trying to convert each column value to each
         type.
         """
-        date_regex = re.compile('^[0-9]{4}-[0-9]{2}-[0-9]{2}$')
-        datetime_regex = re.compile('^[0-9]{4}-[0-9]{2}-[0-9]{2} '
-                                    '[0-9]{2}:[0-9]{2}:[0-9]{2}$')
-        for header in self.headers:
-            column = self[header]
-            possible_types = Counter()
-            for value in column:
-                if value is None or value == '':
-                    continue # None and empty string don't affect type
-                possible_types[str] += 1
-                try:
-                    int_value = int(value)
-                    if unicode(int_value) == unicode(value):
-                        possible_types[int] += 1
-                except ValueError:
-                    pass
-                try:
-                    float_value = float(value)
-                    possible_types[float] += 1
-                except ValueError:
-                    pass
-                if datetime_regex.match(unicode(value)):
-                    possible_types[datetime.datetime] += 1
-                if date_regex.match(unicode(value)):
-                    possible_types[datetime.date] += 1
-            str_count = possible_types[str]
-            types = [float, int, datetime.datetime, datetime.date]
-            result = [type_ for type_ in types
-                            if type_ in possible_types and \
-                               possible_types[type_] == str_count]
-            if not len(result):
-                type_ = str
+        columns = zip(*self._rows)
+        for i, header in enumerate(self.headers):
+            column_types = [int, float, datetime.date, datetime.datetime, str]
+            cant_be = set()
+            try:
+                column = columns[i]
+            except IndexError:
+                self.types[header] = str
             else:
-                type_ = result[-1]
-            self.types[header] = type_
+                types = list(set([type(value) for value in column]) -
+                             set([type(None)]))
+                if len(types) == 1 and types[0] not in (str, unicode):
+                    self.types[header] = types[0]
+                    continue
+                for value in column:
+                    if value == '':
+                        value = None
+                    try:
+                        converted = int(value)
+                        if str(converted) != str(value):
+                            raise ValueError('It is float')
+                    except ValueError:
+                        cant_be.add(int)
+                    except TypeError:
+                        pass  # None should pass
+                    try:
+                        converted = float(value)
+                    except ValueError:
+                        cant_be.add(float)
+                    except TypeError:
+                        pass  # None should pass
+                    if value is not None:
+                        if datetime_regex.match(unicode(value)) is None:
+                            cant_be.add(datetime.datetime)
+                        if date_regex.match(unicode(value)) is None:
+                            cant_be.add(datetime.date)
+                for removed_type in cant_be:
+                    column_types.remove(removed_type)
+                self.types[header] = column_types[0]
 
     def normalize_types(self):
         self._identify_type_of_data()
